use crate::botv2::{
    domain::concour::{check_if_allowed::check_if_allowed, list_concour::list_concour},
    init::{Context, Error},
};
use poise::{
    serenity_prelude::{model::colour, CreateEmbed, CreateEmbedFooter, Mentionable, RoleId},
    CreateReply,
};
use tracing::instrument;

/// List all concour (only for admin)
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    slash_command,
    prefix_command,
    category = "server_config",
    guild_only = true
)]
pub async fn list(ctx: Context<'_>) -> Result<(), Error> {
    let guild = match ctx.guild_id() {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let entity_name = ctx.data().entity_name.clone();
    let footer = CreateEmbedFooter::new(entity_name.clone());
    match check_if_allowed(guild.get(), ctx.author().id.get(), ctx.http()).await {
        Ok(ok) => {
            if !ok {
                let embed = CreateEmbed::new()
                    .title("You are not an admin")
                    .color(colour::Color::RED)
                    .footer(footer);
                if let Err(why) = ctx
                    .send(CreateReply::default().embed(embed).ephemeral(true))
                    .await
                {
                    tracing::error!("Error sending message: {:?}", why);
                }
                return Ok(());
            }
        }
        Err(_) => {
            let embed = CreateEmbed::new()
                .title("You are not an admin")
                .color(colour::Color::RED)
                .footer(footer);
            if let Err(why) = ctx
                .send(CreateReply::default().embed(embed).ephemeral(true))
                .await
            {
                tracing::error!("Error sending message: {:?}", why);
            }
            return Ok(());
        }
    };

    let concour = match list_concour(guild.get()).await {
        Ok(concour) => {
            if concour.is_empty() {
                vec![CreateEmbed::new()
                    .title("No concour found")
                    .color(colour::Color::DARK_GREEN)
                    .footer(footer)]
            } else {
                let mut list = vec![CreateEmbed::new()
                    .title("Concour list")
                    .color(colour::Color::DARK_GREEN)];
                concour.into_iter().for_each(|c| {
                    let mut output = CreateEmbed::new()
                        .title(c.title)
                        .description(c.description)
                        .field("Start date", c.start_date.to_string(), false)
                        .field("Periode", c.periode.to_string(), false)
                        .field("keyword", c.keywords.len().to_string(), true)
                        .footer(footer.clone());
                    if c.role_recompense != 0 {
                        output = output.field(
                            "Role récompense",
                            RoleId::new(c.role_recompense).mention().to_string(),
                            false,
                        );
                    }
                    list.push(output);
                });
                list
            }
        }
        Err(_) => vec![CreateEmbed::new()
            .title("Error while getting concour list")
            .color(colour::Color::RED)
            .footer(footer)],
    };

    let mut builder = CreateReply::default().ephemeral(true);
    builder.embeds = concour;
    if let Err(why) = ctx.send(builder).await {
        tracing::error!("Error sending message: {:?}", why);
    }
    Ok(())
}
