use crate::botv2::{
    domain::concour::{
        check_if_allowed::check_if_allowed,
        stop_concour::{stop_concour, StopConcourError},
    },
    init::{Context, Error},
};
use poise::{
    serenity_prelude::{model::colour, CreateEmbed, CreateEmbedFooter},
    CreateReply,
};
use tracing::instrument;

/// Stop concour (only for admin)
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    slash_command,
    prefix_command,
    category = "server_config",
    guild_only = true
)]
pub async fn stop(ctx: Context<'_>) -> Result<(), Error> {
    let guild = match ctx.guild_id() {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let entity_name = ctx.data().entity_name.clone();
    let mut cron_schedule = ctx.data().scheduler.clone();
    let footer = CreateEmbedFooter::new(entity_name.clone());
    match check_if_allowed(guild.get(), ctx.author().id.get(), ctx.http()).await {
        Ok(ok) => {
            if !ok {
                let embed = CreateEmbed::new()
                    .title("You are not an admin")
                    .color(colour::Color::RED)
                    .footer(footer);
                if let Err(why) = ctx
                    .send(CreateReply::default().embed(embed).ephemeral(true))
                    .await
                {
                    tracing::error!("Error sending message: {:?}", why);
                }
                return Ok(());
            }
        }
        Err(_) => {
            let embed = CreateEmbed::new()
                .title("You are not an admin")
                .color(colour::Color::RED)
                .footer(footer);
            if let Err(why) = ctx
                .send(CreateReply::default().embed(embed).ephemeral(true))
                .await
            {
                tracing::error!("Error sending message: {:?}", why);
            }
            return Ok(());
        }
    };

    let (concour, success) =
        match stop_concour(guild.get(), ctx.channel_id().get(), &mut cron_schedule).await {
            Ok(concour) => {
                if concour.is_none() {
                    (
                        CreateEmbed::new()
                            .title("No concour created")
                            .color(colour::Color::RED),
                        false,
                    )
                } else {
                    let concour = concour.unwrap();
                    let output = CreateEmbed::new()
                        .title(format!("Concour: {}", concour.title))
                        .description(concour.description)
                        .field("Concour mis en pause ", "", false)
                        .color(colour::Color::DARK_GREEN);
                    (output, true)
                }
            }
            Err(err) => (
                match err {
                    StopConcourError::AlreadyStopped => CreateEmbed::new()
                        .title("Concour already Stopped")
                        .color(colour::Color::RED),
                    StopConcourError::DoesntExist => CreateEmbed::new()
                        .title("Concour doesn't exist")
                        .color(colour::Color::RED),
                    StopConcourError::NotScheduled => CreateEmbed::new()
                        .title("Concour not scheduled")
                        .color(colour::Color::RED),
                    _ => CreateEmbed::new()
                        .title("Error while creating concour")
                        .field("Please contact your administrator", "", false)
                        .field("Your concour is possibly not initied correctly", "", false)
                        .color(colour::Color::RED),
                },
                false,
            ),
        };
    let mut builder = CreateReply::default().ephemeral(!success);
    builder = builder.embed(concour.footer(footer));
    if let Err(why) = ctx.send(builder).await {
        tracing::error!("Error sending message: {:?}", why);
    }

    Ok(())
}
