use crate::botv2::{
    domain::server_config::{
        check_if_server_enable_and_admin::check_if_server_enable_and_user_admin,
        enable_feature_auto_meme::enable_feature_meme,
        enable_feature_concour::enable_feature_concour,
    },
    init::{Context, Error},
};
use poise::{
    serenity_prelude::{model::colour, CreateEmbed, CreateEmbedFooter},
    CreateReply,
};
use tracing::instrument;

/// Enable/Disable feature
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    slash_command,
    prefix_command,
    category = "server_config",
    guild_only = true
)]
pub async fn feature(
    ctx: Context<'_>,
    #[description = "Enable/Disable the auto_meme feature"] auto_meme: Option<bool>,
    #[description = "Enable/Disable the concour feature"] concour: Option<bool>,
) -> Result<(), Error> {
    let guild = match ctx.guild_id() {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let entity_name = ctx.data().entity_name.clone();
    let footer = CreateEmbedFooter::new(entity_name.clone());
    match check_if_server_enable_and_user_admin(guild.get(), ctx.author().id.get(), ctx.http())
        .await
    {
        Ok((ok, _)) => {
            if !ok {
                let embed = CreateEmbed::new()
                    .title("You are not an admin")
                    .color(colour::Color::RED)
                    .footer(footer);
                if let Err(why) = ctx
                    .send(CreateReply::default().embed(embed).ephemeral(true))
                    .await
                {
                    tracing::error!("Error sending message: {:?}", why);
                }
                return Ok(());
            }
        }
        Err(_) => {
            let embed = CreateEmbed::new()
                .title("You are not an admin")
                .color(colour::Color::RED)
                .footer(footer);
            if let Err(why) = ctx
                .send(CreateReply::default().embed(embed).ephemeral(true))
                .await
            {
                tracing::error!("Error sending message: {:?}", why);
            }
            return Ok(());
        }
    };
    let answer_auto_meme = match auto_meme {
        Some(enable) => match enable_feature_meme(guild.get(), enable).await {
            Ok(_) => Some(
                CreateEmbed::new()
                    .title(format!("Auto meme feature initialized: {}", enable))
                    .color(colour::Color::DARK_GREEN),
            ),
            Err(_) => Some(
                CreateEmbed::new()
                    .title("Error initializing auto meme feature")
                    .color(colour::Color::RED),
            ),
        },
        None => None,
    };
    let answer_enable_concour = match concour {
        Some(enable) => match enable_feature_concour(guild.get(), enable).await {
            Ok(_) => Some(
                CreateEmbed::new()
                    .title(format!("Concour feature initialized: {}", enable))
                    .color(colour::Color::DARK_GREEN),
            ),
            Err(_) => Some(
                CreateEmbed::new()
                    .title("Error initializing concour feature")
                    .color(colour::Color::RED),
            ),
        },
        None => None,
    };

    let mut builder = CreateReply::default().ephemeral(true);
    if answer_auto_meme.is_none() && answer_enable_concour.is_none() {
        builder = builder.embed(
            CreateEmbed::new()
                .title("No feature initialized")
                .color(colour::Color::RED)
                .footer(footer.clone()),
        );
    }
    if let Some(embed) = answer_auto_meme {
        builder = builder.embed(embed.footer(footer.clone()));
    }
    if let Some(embed) = answer_enable_concour {
        builder = builder.embed(embed.footer(footer));
    }

    if let Err(why) = ctx.send(builder).await {
        tracing::error!("Error sending message: {:?}", why);
    }
    Ok(())
}
