use poise::serenity_prelude::Http;
use serde::{Deserialize, Serialize};
use tracing::{info, instrument};

use crate::{
    db::concour::{Concour, ConcourStatus},
    event::schedule_job::ScheduleJob,
};

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum StartConcourError {
    AlreadyOnGoing,
    DoesntExist,
    KeyWordListEmpty,
    FinishedKeyWordList,
    RoleRecompenseEmpty,
    FindError(String),
    UnknownError(String),
}

#[instrument(level = "info", skip(cron_scheduler, http))]
pub async fn start_concour(
    server_id: u64,
    channel_id: u64,
    cron_scheduler: &mut ScheduleJob,
    http: &Http,
) -> Result<Option<Concour>, StartConcourError> {
    let concour = match Concour::find_by_server_id_channel_id(&server_id, &channel_id).await {
        Ok(list_concour) => list_concour,
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error finding concour");
            return Err(StartConcourError::UnknownError(
                "Error finding concour".to_string(),
            ));
        }
    };

    if concour.is_none() {
        info!("Concour doesn't exist");
        return Err(StartConcourError::DoesntExist);
    }
    let mut concour = concour.unwrap();
    if concour.role_recompense == 0 {
        tracing::warn!("Role recompense is empty");
        return Err(StartConcourError::RoleRecompenseEmpty);
    }
    if concour.keywords.is_empty() {
        tracing::warn!("Keyword list is empty");
        return Err(StartConcourError::KeyWordListEmpty);
    }
    let keyword_len = concour.keywords.len() as u64;
    if concour.index_keyword.gt(&keyword_len) {
        tracing::warn!(keyword_len, concour.index_keyword, "Finished keyword list");
        return Err(StartConcourError::FinishedKeyWordList);
    }
    if concour.status == ConcourStatus::OnGoing {
        tracing::warn!("Concour already OnGoing");
        return Err(StartConcourError::AlreadyOnGoing);
    }
    match cron_scheduler
        .add_concour_cron_job(server_id, channel_id, concour.periode.clone(), http)
        .await
    {
        Ok(_) => {}
        Err(_) => {
            tracing::error!("Error starting cron job");
            return Err(StartConcourError::UnknownError(
                "Error starting cron job".to_string(),
            ));
        }
    }
    concour.status = ConcourStatus::OnGoing;

    match concour.update().await {
        Ok(_) => {}
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error updating concour");
            return Err(StartConcourError::UnknownError(
                "Error updating concour".to_string(),
            ));
        }
    }
    Ok(Some(concour))
}

#[instrument(level = "info")]
pub async fn start_concour_step2(
    server_id: u64,
    channel_id: u64,
    message_id: u64,
) -> Result<(), StartConcourError> {
    let concour = match Concour::find_by_server_id_channel_id(&server_id, &channel_id).await {
        Ok(list_concour) => list_concour,
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error finding concour");
            return Err(StartConcourError::UnknownError(
                "Error finding concour".to_string(),
            ));
        }
    };

    if concour.is_none() {
        info!("Concour doesn't exist");
        return Err(StartConcourError::DoesntExist);
    }
    let mut concour = concour.unwrap();
    concour.last_message_id = Some(message_id);
    match concour.update().await {
        Ok(_) => {}
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error updating concour");
            return Err(StartConcourError::UnknownError(
                "Error updating concour".to_string(),
            ));
        }
    }
    Ok(())
}
