use serde::{Deserialize, Serialize};
use tracing::{info, instrument};

use crate::{
    db::concour::{Concour, ConcourStatus},
    event::schedule_job::{ScheduleJob, StopScheduleJob},
};

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum StopConcourError {
    DoesntExist,
    AlreadyStopped,
    NotScheduled,
    FindError(String),
    UnknownError(String),
}

#[instrument(level = "info", skip(cron_scheduler))]
pub async fn stop_concour(
    server_id: u64,
    channel_id: u64,
    cron_scheduler: &mut ScheduleJob,
) -> Result<Option<Concour>, StopConcourError> {
    let concour = match Concour::find_by_server_id_channel_id(&server_id, &channel_id).await {
        Ok(list_concour) => list_concour,
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error finding concour");
            return Err(StopConcourError::UnknownError(
                "Error finding concour".to_string(),
            ));
        }
    };

    if concour.is_none() {
        info!("Concour doesn't exist");
        return Err(StopConcourError::DoesntExist);
    }
    let mut concour = concour.unwrap();

    if concour.status == ConcourStatus::Paused {
        return Err(StopConcourError::AlreadyStopped);
    }
    match cron_scheduler
        .stop_scheduled_job(server_id, channel_id)
        .await
    {
        Ok(_) => {}
        Err(e) => {
            tracing::error!(err = e.to_string(), "Error stopping cron job");
            match e {
                StopScheduleJob::JobNotFound => {
                    tracing::error!("Job not found");
                    return Err(StopConcourError::NotScheduled);
                }
                StopScheduleJob::RemoveFailed => {
                    tracing::error!("Remove failed");
                    return Err(StopConcourError::UnknownError(
                        "Error stopping cron job".to_string(),
                    ));
                }
            }
        }
    }
    concour.status = ConcourStatus::Paused;
    concour.last_message_id = None;

    match concour.update().await {
        Ok(_) => {}
        Err(err) => {
            tracing::error!(error = err.to_string(), "Error updating concour");
            return Err(StopConcourError::UnknownError(
                "Error updating concour".to_string(),
            ));
        }
    }
    Ok(Some(concour))
}
