use crate::img::config_file::{ConfigFile, KeyWordItem};
use poise::serenity_prelude::{model::colour, CreateAttachment, CreateEmbed, CreateEmbedFooter};
use rand::Rng;
use tokio::fs::File;
use tracing::error;
use tracing::instrument;

pub enum AnswerError {
    ConfigGlobal,
    ConfigImgGlobal,
    KeyWordItem,
    UnknownError(String),
}

pub enum AnswerResult {
    Embed(Vec<CreateEmbed>, Vec<CreateAttachment>),
    Animated(Vec<CreateAttachment>),
}

#[instrument(level = "info", skip(config_img, base_path))]
pub async fn answer_meme(
    user_id: u64,
    guild_id: u64,
    keyword: String,
    base_path: String,
    config_img: ConfigFile,
    bot_entity: String,
) -> Result<AnswerResult, AnswerError> {
    let footer = CreateEmbedFooter::new(bot_entity);

    let folder_container = match config_img
        .keyword
        .iter()
        .find(|keyword_under_search| keyword_under_search.does_value_match(keyword.clone()))
    {
        Some(keyword_matching) => {
            let keyword_path = match keyword_matching.path.len() {
                0 => keyword_matching.path[0].clone(),
                _ => {
                    let id: usize = {
                        let mut rng = rand::thread_rng();
                        rng.gen_range(0..keyword_matching.path.len())
                    };
                    keyword_matching.path[id].clone()
                }
            };
            keyword_path.clone()
        }
        None => {
            return Ok(AnswerResult::Embed(
                vec![CreateEmbed::new()
                    .title("No match found")
                    .footer(footer)
                    .color(colour::Color::RED)],
                vec![],
            ));
        }
    };
    let path = format!("{}/{}", base_path, folder_container);
    let file_folder = KeyWordItem::output_folder_content(path.clone());
    let id_rand: usize = {
        let mut rng = rand::thread_rng();
        rng.gen_range(0..file_folder.len())
    };
    let filename = match file_folder.get(id_rand) {
        Some(file) => file.file_name().to_str().unwrap(),
        None => {
            error!(folder = path, "Couldn't find a file to send");
            return Ok(AnswerResult::Embed(
                vec![CreateEmbed::new()
                    .title("Couldn't find a file to send, please contact your administrator")
                    .footer(footer)
                    .color(colour::Color::RED)],
                vec![],
            ));
        }
    };
    let file_path = format!("{}/{}", path, filename);
    let file = match File::open(file_path.clone()).await {
        Ok(file) => file,
        Err(e) => {
            error!(
                folder = file_path,
                error = e.to_string(),
                "Couldn't open file"
            );
            return Ok(AnswerResult::Embed(
                vec![CreateEmbed::new()
                    .title("Couldn't find file, please contact your administrator")
                    .footer(footer)
                    .color(colour::Color::RED)],
                vec![],
            ));
        }
    };
    let attachment = match CreateAttachment::file(&file, filename).await {
        Ok(attachment) => attachment,
        Err(e) => {
            error!(
                folder = file_path,
                error = e.to_string(),
                "Couldn't create attachment"
            );
            return Ok(AnswerResult::Embed(
                vec![CreateEmbed::new()
                    .title("Couldn't create attachment, please contact your administrator")
                    .footer(footer)
                    .color(colour::Color::RED)],
                vec![],
            ));
        }
    };
    if filename.ends_with(".mp3") || filename.ends_with(".mp4") {
        return Ok(AnswerResult::Animated(vec![attachment]));
    }
    let embed = CreateEmbed::new()
        .title(keyword)
        .footer(footer)
        .image(format!("attachment://{}", filename));
    Ok(AnswerResult::Embed(vec![embed], vec![attachment]))
}
