use super::init::DB;
use serde::{Deserialize, Serialize};
use std::fmt;
use surrealdb::opt::Resource;
use surrealdb::sql::Thing;
use tracing::instrument;
use utoipa::ToSchema;

const CONCOUR: &str = "concour";

#[derive(Debug, Serialize, Deserialize, Clone, ToSchema, PartialEq, Eq)]
pub enum ConcourStatus {
    Created,
    Paused,
    OnGoing,
    Finished,
}

impl fmt::Display for ConcourStatus {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{:?}", self)
    }
}

#[derive(Debug, Serialize, Deserialize, Clone, ToSchema)]
pub struct ConcourRecord {
    pub id: Thing,
    pub server_id: u64,
    pub channel_id: u64,
}

#[derive(Debug, Serialize, Deserialize, Clone, ToSchema)]
pub struct ConcourWinner {
    pub user_id: u64,
    pub date: chrono::DateTime<chrono::Utc>,
    pub keyword: String,
}

#[derive(Debug, Serialize, Deserialize, Clone, ToSchema)]
pub struct Concour {
    pub server_id: u64,
    pub channel_id: u64,
    pub title: String,
    pub description: String,
    pub start_date: chrono::DateTime<chrono::Utc>,
    pub periode: String,
    pub role_recompense: u64,
    pub keywords: Vec<String>,
    pub banner: Option<String>,
    pub index_keyword: u64,
    pub status: ConcourStatus,
    pub winner: Vec<ConcourWinner>,
    pub last_message_id: Option<u64>,
}

impl Default for Concour {
    fn default() -> Self {
        Self {
            server_id: 0,
            channel_id: 0,
            title: String::new(),
            description: String::new(),
            start_date: chrono::Utc::now(),
            periode: "0 0 17 * * * *".to_string(),
            role_recompense: 0,
            keywords: Vec::new(),
            banner: None,
            index_keyword: 0,
            status: ConcourStatus::Created,
            winner: Vec::new(),
            last_message_id: None,
        }
    }
}

impl Concour {
    pub fn new(server_id: u64, channel_id: u64) -> Result<Self, surrealdb::Error> {
        Ok(Self {
            server_id,
            channel_id,
            title: String::new(),
            description: String::new(),
            start_date: chrono::Utc::now(),
            periode: "0 0 17 * * * *".to_string(),
            role_recompense: 0,
            keywords: Vec::new(),
            banner: None,
            index_keyword: 0,
            status: ConcourStatus::Created,
            winner: Vec::new(),
            last_message_id: None,
        })
    }
    #[instrument(level = "info")]
    pub async fn create(&self) -> Result<(), surrealdb::Error> {
        match DB.create(Resource::from(CONCOUR)).content(&self).await {
            Ok(_) => {}
            Err(e) => {
                return Err(e);
            }
        };
        Ok(())
    }
    #[instrument(level = "info")]
    pub async fn update(&self) -> Result<bool, surrealdb::Error> {
        let id = match Concour::get_records(&self.server_id, &self.channel_id).await {
            Ok(Some(concour)) => concour.id,
            Ok(None) => return Ok(false),
            Err(e) => {
                tracing::error!("Error getting concour id: {:?}", e);
                return Err(e);
            }
        };
        match DB
            .update::<Option<Concour>>((CONCOUR.to_string(), id))
            .merge(self)
            .await
        {
            Ok(concour) => {
                if concour.is_none() {
                    return Ok(false);
                }
                tracing::info!("Concour updated: {:?}", concour);
                Ok(true)
            }
            Err(e) => {
                tracing::error!("Error updating concour: {:?}", e);
                Err(e)
            }
        }
    }
    #[instrument(level = "info")]
    pub async fn get_records(
        server_id: &u64,
        channel_id: &u64,
    ) -> Result<Option<ConcourRecord>, surrealdb::Error> {
        let sql = format!(
            "SELECT id,server_id,channel_id FROM {} WHERE server_id = {} AND channel_id = {}",
            CONCOUR, server_id, channel_id
        );
        let mut results = match DB.query(&sql).await {
            Ok(results) => results,
            Err(e) => {
                return Err(e);
            }
        };
        let concour: ConcourRecord = match results.take(0) {
            Ok(Some(concour)) => concour,
            Ok(None) => {
                return Ok(None);
            }
            Err(e) => {
                return Err(e);
            }
        };
        Ok(Some(concour))
    }
    #[instrument(level = "info")]
    pub async fn find_by_server_id_channel_id(
        server_id: &u64,
        channel_id: &u64,
    ) -> Result<Option<Concour>, surrealdb::Error> {
        let sql = format!(
            "SELECT * FROM {} WHERE server_id = {} AND channel_id = {}",
            CONCOUR, server_id, channel_id
        );
        let mut results = match DB.query(&sql).await {
            Ok(results) => results,
            Err(e) => {
                return Err(e);
            }
        };
        let concour: Concour = match results.take(0) {
            Ok(Some(concour)) => concour,
            Ok(None) => {
                return Ok(None);
            }
            Err(e) => {
                return Err(e);
            }
        };
        Ok(Some(concour))
    }
    pub async fn find_by_server_id(server_id: &u64) -> Result<Vec<Concour>, surrealdb::Error> {
        let sql = format!("SELECT *  FROM {} WHERE server_id = {}", CONCOUR, server_id);
        let mut results = match DB.query(&sql).await {
            Ok(results) => results,
            Err(e) => {
                return Err(e);
            }
        };
        let mut concours = Vec::new();
        while let Ok(Some(concour)) = results.take(0) {
            concours.push(concour);
        }
        Ok(concours)
    }
    #[instrument(level = "info")]
    pub async fn find_by_status(status: &ConcourStatus) -> Result<Vec<Concour>, surrealdb::Error> {
        let sql = format!("SELECT *  FROM {} WHERE status = '{}'", CONCOUR, status);
        let mut results = match DB.query(&sql).await {
            Ok(results) => results,
            Err(e) => {
                return Err(e);
            }
        };
        let mut concours = Vec::new();
        while let Ok(Some(concour)) = results.take(0) {
            concours.push(concour);
        }
        Ok(concours)
    }
}
